package org.jboss.internal.soa.esb.message.format.serialized.body.content;

import org.jboss.internal.soa.esb.message.format.serialized.BodyImpl;
import org.jboss.soa.esb.message.body.content.InvalidPayloadException;
import org.jboss.soa.esb.message.body.content.Payload;
import org.jboss.soa.esb.message.body.content.TextBody;

/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author mark.little@jboss.com
 */

/**
 * Similar to JMS, the Message payload only contains a String.
 */

public class TextBodyImpl extends BodyImpl implements TextBody
{
	public static final String ELEMENT_NAME = "org.jboss.soa.esb.message.payload.text";

	private static final long serialVersionUID = 0xdeadbeef;

	public TextBodyImpl (BodyImpl b)
	{
		super(b);
		
		add(Payload.CONTENT_TYPE, Payload.TEXT_BODY);
	}
	
	/**
	 * Add the Text element to the Body.
	 * 
	 * @param value
	 *            the String to add.
	 */

	public void setText(String value)
	{
		if (value == null)
			throw new IllegalArgumentException();

		super.add(ELEMENT_NAME, value);
	}

	/**
	 * Obtain the Text from the Body.
	 * 
	 * @return the String.
	 */

	public String getText() throws InvalidPayloadException
	{
		try
		{
			Object ret = super.get(ELEMENT_NAME);

			if (ret == null)
				throw new InvalidPayloadException();

			return (String) ret;
		}
		catch (ClassCastException ex)
		{
			throw new InvalidPayloadException(ex);
		}
	}
}