package org.jboss.internal.soa.esb.message.format.xml.body.content;

import java.io.Serializable;

import org.jboss.internal.soa.esb.message.format.xml.BodyImpl;
import org.jboss.soa.esb.message.body.content.InvalidPayloadException;
import org.jboss.soa.esb.message.body.content.ObjectBody;
import org.jboss.soa.esb.message.body.content.Payload;

/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author mark.little@jboss.com
 */

/**
 * Similar to JMS, the Message payload only contains a serializable Object.
 */

public class ObjectBodyImpl extends BodyImpl implements ObjectBody
{
	public static final String ELEMENT_NAME = "org.jboss.soa.esb.message.body.object.content";
	
	public ObjectBodyImpl (BodyImpl body)
	{
		super(body);
		
		add(Payload.CONTENT_TYPE, Payload.OBJECT_BODY);
	}
	
	/**
	 * Add the Object element to the Body.
	 * 
	 * @param value
	 *            the Object to add.
	 */

	public void setObject(Serializable value)
	{
		if (value == null)
			throw new IllegalArgumentException();
		else
			super.add(ELEMENT_NAME, value);
	}

	/**
	 * Obtain the Object from the Body.
	 * 
	 * @return the Object.
	 */

	public Serializable getObject() throws InvalidPayloadException
	{
		try
		{
			Object ret = super.get(ELEMENT_NAME);

			if (ret == null)
				throw new InvalidPayloadException();

			return (Serializable) ret;
		}
		catch (ClassCastException ex)
		{
			// not serializable

			throw new InvalidPayloadException(ex);
		}
	}

}