/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.soa.esb.actions.naming.strategy;

import org.jboss.soa.esb.message.Message;

/**
 * Abstract base class that handles common concerns for file naming.
 * 
 * @author Derek Adams
 */
public abstract class AbstractFileNamingStrategy implements FileNamingStrategy {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jboss.soa.esb.actions.naming.strategy.FileNamingStrategy#process(java.lang.String,
	 *      org.jboss.soa.esb.message.Message)
	 */
	public abstract String process(String filename, Message message);

	/**
	 * Gets the filename without the suffix.
	 * 
	 * @param filename
	 * @return String
	 */
	public String getFilenameWithoutSuffix(String filename) {
		int lastDot = filename.lastIndexOf('.');
		if (lastDot < 1) {
			return filename;
		} else {
			return filename.substring(0, lastDot);
		}
	}

	/**
	 * Gets the filename without the suffix.
	 * 
	 * @param filename
	 * @return String
	 */
	public String getSuffix(String filename) {
		int lastDot = filename.lastIndexOf('.');
		if (lastDot == -1) {
			return filename;
		} else {
			return filename.substring(lastDot + 1);
		}
	}
}