/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.soa.esb.lifecycle;

/**
 * Interface representing a lifecycle resource factory.
 */
public interface LifecycleResourceFactory<R>
{
    /**
     * Create a resource object which will be associated with the specified lifecycle identity.
     * @param lifecycleIdentity The associated lifecycle identity.
     * @return The lifecycle resource
     * @throws LifecycleResourceException for errors during construction.
     */
    public R createLifecycleResource(final String lifecycleIdentity)
        throws LifecycleResourceException ;
    
    /**
     * Destroy a resource object which is associated with the specified lifecycle identity.
     * @param resource The lifecycle resource.
     * @param lifecycleIdentity The associated lifecycle identity.
     * @return The lifecycle resource.
     * @throws LifecycleResourceException for errors during destroy.
     */
    public void destroyLifecycleResource(final R resource, final String lifecycleIdentity)
        throws LifecycleResourceException ;
}
