/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.soa.esb.message.tests;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URI;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.jboss.internal.soa.esb.message.format.serialized.MessageImpl;
import org.jboss.soa.esb.couriers.FaultMessageException;
import org.jboss.soa.esb.listeners.message.errors.Factory;
import org.jboss.soa.esb.message.Message;
import org.jboss.soa.esb.message.format.MessageFactory;
import org.jboss.soa.esb.message.format.MessageType;
import org.jboss.soa.esb.util.Util;

/**
 * Unit tests for the Class class.
 * 
 * @author Mark Little
 */

public class FaultUnitTest extends TestCase
{
	private Logger log = Logger.getLogger( FaultUnitTest.class );
	
	public void testSerializeFault ()
	{
		Message msg = MessageFactory.getInstance().getMessage(
				MessageType.JAVA_SERIALIZED);

		assertEquals((msg != null), true);

		try
		{
			URI code = new URI("urn:code");
		
			msg.getFault().setCode(code);
			msg.getFault().setReason("because");
			
			assertEquals(msg.getFault().getCode(), code);
			assertEquals(msg.getFault().getReason(), "because");

			ByteArrayOutputStream s = new ByteArrayOutputStream();
			ObjectOutputStream o = new ObjectOutputStream(s);

			o.writeObject(msg);
			o.close();

			ByteArrayInputStream is = new ByteArrayInputStream(s.toByteArray());
			ObjectInputStream io = new ObjectInputStream(is);

			MessageImpl nImpl = (MessageImpl) io.readObject();

			assertEquals(nImpl.getFault().getReason(), "because");
			
			o.close();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			
			fail(ex.toString());
		}
	}

	public void testXMLFault ()
	{
		Message msg = MessageFactory.getInstance().getMessage(
				MessageType.JBOSS_XML);

		assertEquals((msg != null), true);

		try
		{
			URI code = new URI("urn:code");
			
			msg.getFault().setCode(code);
			msg.getFault().setReason("because");
			
			assertEquals(msg.getFault().getCode(), code);
			assertEquals(msg.getFault().getReason(), "because");
		
                        final String documentAsString = (String)Util.serialize(msg) ;

                        log.debug("Message looks like: " + documentAsString);
                        
                        final Message nImpl = Util.deserialize(documentAsString);
			
			assertEquals(nImpl.getFault().getReason(), "because");
		}
		catch (Exception ex)
		{
			fail(ex.toString());
		}
	}

	public void testSerializableFaultMessageException () throws IOException
	{
		final Message message = MessageFactory.getInstance().getMessage(MessageType.JBOSS_XML) ;
		final FaultMessageException exception = new FaultMessageException("error message", URI.create(Factory.UNEXPECTED_ERROR), message) ;
		final ByteArrayOutputStream baos = new ByteArrayOutputStream() ;
		final ObjectOutputStream oos = new ObjectOutputStream(baos) ;
		oos.writeObject(exception) ;
	}
}
