/*
 * JBoss, Home of Professional Open Source Copyright 2009, Red Hat Middleware
 * LLC, and individual contributors by the @authors tag. See the copyright.txt
 * in the distribution for a full listing of individual contributors.
 * 
 * This is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA, or see the FSF
 * site: http://www.fsf.org.
 */
package org.jboss.soa.esb.services.jbpm.actionhandlers;

import static org.jboss.soa.esb.services.jbpm.Constants.REPLY_TO;
import static org.jboss.soa.esb.services.jbpm.Constants.FAULT_TO;
import static org.jboss.soa.esb.services.jbpm.Constants.ESB_MESSAGE_ID;

import java.net.URI;

import org.jboss.soa.esb.message.Message;
import org.jbpm.context.exe.ContextInstance;

/**
 * Utils class for methods that are common to different EsbAction handlers
 * and notifiers.
 * 
 * @author <a href="mailto:dbevenius@jboss.com">Daniel Bevenius</a>
 *
 */
public final class ActionUtil
{
    private ActionUtil()
    {
    }
    
    public static boolean isReplyToOrFaultToSet(final ContextInstance ci)
    {
        return ci.getVariable(REPLY_TO) != null || ci.getVariable(FAULT_TO) != null;
    }
    
    public static void setRelatesToMessageId(final ContextInstance ci, final Message message)
    {
        final URI esbMessageId = (URI) ci.getVariable(ESB_MESSAGE_ID);
        message.getHeader().getCall().setRelatesTo(esbMessageId);
    }
    
}
